<html>
    <head>
        <title>Changme Default Credential Report</title>
        <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.1.1/jquery.js" integrity="sha256-16cdPddA6VdVInumRGo6IbivbERE8p7CQR3HzTBuELA=" crossorigin="anonymous"></script>
        <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js" integrity="sha384-Tc5IQib027qvyjSMfHjOMaLkfuWVxZxUPnCJA7l2mCWNIpG9mGCD8wGNIcPD7Txa" crossorigin="anonymous"></script>
        <link href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-BVYiiSIFeK1dGmJRAkycuHAHRg32OmUcww7on3RYdg4Va+PmSTsz/K68vbdEjh4u" crossorigin="anonymous">
        <style type=text/css>
            .row { border-top: 1px solid #666; }
            ul { padding-left: 0px; }
            ul li { list-style: none; }
            ul li a { word-wrap: break-word; }
        </style>
    </head>
    <body>
        <div class="container">
            <div class="row">
                <div class="col-sm-12 center">
                    <h1>Changeme</h1>
                    <ul>
                        <li><strong>Found {{ found|length }} credentials</strong></li>
                        <li>Scan: {{ cli }}</li>
                        <li>Report Timestamp: {{ timestamp }}</li>
                    </ul>
                </div>
            </div>
{% for cred in found %}
            <div class="row">
                <div class="col-sm-3">
                    <ul>
                        <li>Service: {{ cred['name'] }}</li>
                        <li>Username: {{ cred['username'] if cred['username'] != None }}</li>
                        <li>Password: {{ cred['password'] if cred['password'] != None }}</li>
                        {% if 'http' in cred['target'].protocol %}
                        <li><a href="{{ cred['target'] }}">{{ cred['target'] }}</a></li>
                        {% else %}
                        <li>{{ cred['target'] }}</li>
                        {% endif %}
                    </ul>
                </div>
                <div class="col-sm-9">
                    {% if 'http' in cred['target'].protocol %}
                    <img alt="{{ cred['url'] }}" src="data:image/png;base64,{{ cred['evidence'] }}" />
                    {% else %}
                    {{ cred['evidence'] }}
                    {% endif %}
                </div>
            </div>
{% endfor %}
        </div>
    </body>
</html>
